//
// CrcList.cpp
// asommers 12-17-96
// asommers 8-5-98
//
// copyright 1998, origin systems
//

#include "First.h"
#include "CrcList.h"

#include "Iff.h"

//
// CrcNode Implementation
//

CrcList::CrcNode::CrcNode (u_int32 new_crc, const char* new_name)
{
	next = NULL;

	crc  = new_crc;
	name = strdup (new_name);
}

CrcList::CrcNode::~CrcNode (void)
{
	if (name)
	{
		delete [] name;
		name = NULL;
	}

	next = 0;
}

//
// CrcList Implementation
//

CrcList::CrcList (void)
{
	list     = NULL;
	numNodes = 0;
}

CrcList::~CrcList (void)
{
	destroy ();
}

void CrcList::destroy (void)
{
	CrcNode* iterator = list;

	while ( iterator != NULL )
	{
		CrcNode* tempPtr = iterator;
		iterator = iterator->next;
		delete tempPtr;
		numNodes--;
	}

	FATAL (numNodes != 0, ("CrcList::~CrcList - numNodes != 0; more crc nodes than were deleted!"));

	list = NULL;
}

void CrcList::addCrc (u_int32 new_crc, const char* new_name)
{
	CrcNode* node = NEW CrcNode (new_crc, new_name);

	node->next = list;
	list = node;

	numNodes++;
}

const char* CrcList::getName (u_int32 crc)
{
	CrcNode* iterator = list;

	while (iterator && iterator->crc != crc)
		iterator = iterator->next;

	return iterator ? iterator->name : 0;
}

void CrcList::load (Iff& iff)
{
	destroy ();

	iff.seekForm (TAG (C,R,C,S));

		iff.seekChunk (TAG (D,A,T,A));

			int n = iff.read_int32 ();

			int i;
			for (i = 0; i < n; i++)
			{
				u_int32 crc = iff.read_u_int32 ();
				char* name  = iff.read_string ();

				addCrc (crc, name);

				delete [] name;
			}

		iff.exitChunk ();

	iff.exitForm ();
}

void CrcList::save (Iff& iff)
{
	iff.appendForm (TAG (C,R,C,S));

		iff.appendChunk (TAG (D,A,T,A));

			Iff__appendToChunk (iff, numNodes);

			CrcNode* iterator;
			for (iterator = list; iterator; iterator = iterator->next)
			{
				Iff__appendToChunk (iff, iterator->crc);
				iff.appendToChunk (iterator->name, strlen (iterator->name)+1);
			}

	iff.exitForm ();
}
